# CASU_SET_PREFIX
#---------------
# Sets and the directory prefix for the package installation. If no
# directory prefix was given on the command line the default prefix
# is appended to the configure argument list and thus passed to
# the subdirs configure.
AC_DEFUN([CASU_SET_PREFIX],
[
  unset CDPATH
  # make cwd the default for the installation
  AC_PREFIX_DEFAULT(${CASUDIR:-$PWD})

  if test "x$prefix" = "xNONE"; then
    prefix=$ac_default_prefix
    ac_configure_args="$ac_configure_args --prefix $prefix"
    if test -d $PWD/bin; then
        rm -rf $PWD/bin
    fi
    mkdir $PWD/bin
  fi
])

# CASU_CHECK_CFITSIO
#------------------
# Checks for the cfitsio library and header files.
AC_DEFUN([CASU_CHECK_CFITSIO],
[
    AC_MSG_CHECKING([for cfitsio])

    casu_cfitsio_check_header="fitsio.h"
    # FIXME: Check first for the dynamic library...
    casu_cfitsio_check_lib="libcfitsio.a"

    casu_cfitsio_dir=""
    casu_cfitsio_incdirs=""
    casu_cfitsio_libdirs=""
    casu_cfitsio_includes=""
    casu_cfitsio_libraries=""

    # Get the CFITSIO directory from the --with-cfitsio CLI option, or
    # else fall back on the environment variable $CFITSIODIR
    AC_ARG_WITH(cfitsio,
                AC_HELP_STRING([--with-cfitsio],
                               [location where cfitsio is installed]),
                [
                    casu_cfitsio_dir=$withval
                ],
                [
                    casu_cfitsio_dir=$CFITSIODIR
                ])

    # Check for the cfitsio includes, either in include/, include/cfitsio/
    # or include/libcfitsio0/
    casu_cfitsio_incdirs="$casu_cfitsio_dir/include"
    CASU_FIND_FILE($casu_cfitsio_check_header, $casu_cfitsio_incdirs, casu_cfitsio_includes)
    if test x"$casu_cfitsio_includes" = xno; then
      # include/cfitsio/ is seen on a 64-bit Fedora 10
      casu_cfitsio_incdirs="$casu_cfitsio_dir/include/cfitsio"
      CASU_FIND_FILE($casu_cfitsio_check_header, $casu_cfitsio_incdirs, casu_cfitsio_includes)
      if test x"$casu_cfitsio_includes" = xno; then
        # include/libcfitsio0/ is seen on a 64-bit Suse
        casu_cfitsio_incdirs="$casu_cfitsio_dir/include/libcfitsio0"
        CASU_FIND_FILE($casu_cfitsio_check_header, $casu_cfitsio_incdirs, casu_cfitsio_includes)
      fi
    fi

    # Check for the cfitsio library, either in lib64/, lib32/ or lib/
    casu_cfitsio_libdirs="$casu_cfitsio_dir/lib64"
    CASU_FIND_FILE($casu_cfitsio_check_lib, $casu_cfitsio_libdirs, casu_cfitsio_libraries)

    if test x"$casu_cfitsio_libraries" = xno; then
      casu_cfitsio_libdirs="$casu_cfitsio_dir/lib32"
      CASU_FIND_FILE($casu_cfitsio_check_lib, $casu_cfitsio_libdirs, casu_cfitsio_libraries)
      if test x"$casu_cfitsio_libraries" = xno; then
        casu_cfitsio_libdirs="$casu_cfitsio_dir/lib"
        CASU_FIND_FILE($casu_cfitsio_check_lib, $casu_cfitsio_libdirs, casu_cfitsio_libraries)
      fi
    fi

    if test x"$casu_cfitsio_includes" = xno || test x"$casu_cfitsio_libraries" = xno; then
        AC_MSG_ERROR([cfitsio was not found on your system. Please check!])
    else
        AC_MSG_RESULT([libraries $casu_cfitsio_libraries, headers $casu_cfitsio_includes])
        # Attempt to check the version via CFITSIO_VERSION in the include file
        casu_cfitsio_check_vers=`perl -nle 's/^#\s*define\s+CFITSIO_VERSION\s+\b// and s/\s.*//, print' $casu_cfitsio_includes/fitsio.h`
       if test -z "$casu_cfitsio_check_vers" ; then
         # Attempt to check the version by checking the include files
         casu_cfitsio_check_vers=`grep "Version Info: This file is distributed with version 2.510 of CFITSIO" $casu_cfitsio_includes/fitsio.h`
         if test -z "$casu_cfitsio_check_vers" ; then
             AC_MSG_WARN([cfitsio version seems to be different from 2.510 and less than 3.X.])
	 else
             AC_MSG_WARN([cfitsio version seems to be 2.510])
         fi
       else
          CFLAGS="-D_LARGEFILE_SOURCE=1 -D_FILE_OFFSET_BITS=64 $CFLAGS"
       fi
    fi

    # Set up the symbols
    CFITSIO_INCLUDES="-I$casu_cfitsio_includes"
    CFITSIO_LDFLAGS="-L$casu_cfitsio_libraries"
    LIBCFITSIO="-lcfitsio"

    AC_SUBST(CFITSIO_INCLUDES)
    AC_SUBST(CFITSIO_LDFLAGS)
    AC_SUBST(LIBCFITSIO)
])

# CASU_CHECK_WCS
#--------------
# Checks for the wcs library and header files.
AC_DEFUN([CASU_CHECK_WCS],
[
    AC_MSG_CHECKING([for wcs])

    casu_wcs_check_header="wcslib/wcslib.h"
    casu_wcs_check_lib="libwcs.a"

    casu_wcs_includes=""
    casu_wcs_libraries=""

    AC_ARG_WITH(wcs,
                AC_HELP_STRING([--with-wcs],
                               [location where wcs is installed]),
                [
                    casu_with_wcs_includes=$withval/include
                    casu_with_wcs_libs=$withval/lib
                ])

    # Check for the wcs includes
    if test -z "$casu_with_wcs_includes"; then
        test -n "$WCSDIR" && casu_wcs_incdirs="$WCSDIR/include"
    else
        casu_wcs_incdirs="$casu_with_wcs_includes"
    fi
    CASU_FIND_FILE($casu_wcs_check_header, $casu_wcs_incdirs, casu_wcs_includes)

    # Check for the wcs library
    if test -z "$casu_with_wcs_libs"; then
        test -n "$WCSDIR" && casu_wcs_libdirs="$WCSDIR/lib"
    else
        casu_wcs_libdirs="$casu_with_wcs_libs"
    fi
    CASU_FIND_FILE($casu_wcs_check_lib, $casu_wcs_libdirs, casu_wcs_libraries)

    if test x"$casu_wcs_includes" = xno || test x"$casu_wcs_libraries" = xno; then
        AC_MSG_WARN([wcs was not found on your system.])
    else
        AC_MSG_RESULT([libraries $casu_wcs_libraries, headers $casu_wcs_includes])
        # Attempt to check the version by checking the include files
#        casu_wcs_check_vers4=`grep "WCSLIB 4.[3-9] - an implementation of the FITS WCS standard" $casu_wcs_includes/wcslib/wcslib.h`
#        if test -z "$casu_wcs_check_vers4" ; then
#            casu_wcs_check_vers5=`grep "WCSLIB [5-9].[0-9] - an implementation of the FITS WCS standard" $casu_wcs_includes/wcslib/wcslib.h`
#            if test -z "$casu_wcs_check_vers5" ; then
#                AC_MSG_WARN([wcs version seems to be older than 4.3])
#            fi
#        fi
        AC_DEFINE_UNQUOTED(CASU_WCS_INSTALLED, 1, [Defined if WCS is available])
        # Set up the symbols
        WCS_INCLUDES="-I$casu_wcs_includes/wcslib"
        WCS_LDFLAGS="-L$casu_wcs_libraries"
        LIBWCS="-lwcs"

        AC_SUBST(WCS_INCLUDES)
        AC_SUBST(WCS_LDFLAGS)
        AC_SUBST(LIBWCS)
    fi

])

# CASU_FIND_FILE(file, directories, variable)
#------------------------------------------
# Search for file in directories. Set variable to the first location
# where file was found, if file is not found at all variable is set to NO.
AC_DEFUN([CASU_FIND_FILE],
[
    $3=no

    for i in $2; do
        for j in $1; do

            echo "configure: __oline__: $i/$j" >&AC_FD_CC

            if test -r "$i/$j"; then
                echo "taking that" >&AC_FD_CC
                $3=$i
                break 2
            fi
        done
    done
])
